const columnify = require('columnify')
const doctrine = require('doctrine')
const minimist = require('minimist')
const path = require('path')

const jsdocRegex = /\/\*\*[\s\S]*?\*\//

let { argv } = process

const executorPath = path.parse(argv.shift())
const applicationPath = path.parse(argv.shift())

const flags = minimist(argv)
const commands = flags._
delete flags._
usedCommands = []

const walk = property => {
  if (commands.length === 0 || typeof property !== 'object') return property

  const usedCommand = commands.shift()
  usedCommands.push(usedCommand)
  return walk(property[usedCommand])
}

const help = (property, err) => {
  if (err) console.log(err.message)
  console.log()

  const commandPath = [applicationPath.name].concat(usedCommands).join(' ')

  console.log(`usage: ${commandPath} <command> [<args>]\n`)

  switch (typeof property) {
    case 'function':
      const jsdocMatch = property.toString().match(jsdocRegex)

      if (!jsdocMatch) break

      const doc = doctrine.parse(jsdocMatch[0], { unwrap: true })

      if (doc.description) console.log(`${doc.description}\n`)

      const params = doc.tags.filter(tag => tag.title === 'param').reduce((data, tag, i) => {
        data[`${i + 1}. ${tag.name}`] = tag.description
        return data
      }, {})

      if (params) {
        console.log('Parameters:')
        console.log(columnify(params, {
          columnSplitter: '\t',
          showHeaders: false
        }))
      }

      break

    case 'object':
      console.log('possible values:')
      Object.keys(property).forEach(key => console.log(`\t${key}`))
      break
  }
}

const execute = property => {
  switch (typeof property) {
    case 'function':
      return property(...commands, flags).catch(err => {
        help(property, err)
        process.exit(1)
      })

    case 'object':
      help(property)
      process.exit(1)
  }
}

module.exports = {
  walk,
  help,
  execute
}
